/* -----------------------------------------------------------------------
 FileManager.java
 Copyright (C) 2003  Nue

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
----------------------------------------------------------------------- */
/*
 * Created on 2003/02/26
 *
 * File Name: FileManager.java
 * Encoding : EUC-JP
 * LineDelimiter: LF
 * 
 */
package jp.co.jlight;

import java.io.*;
import java.util.*;
import java.text.MessageFormat;

import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
/**
 * ե륯饹
 */
public class FileManager {

	private Shell shell;
	private String fileName = "";
	private String fullFileName = "";
	private String path = "";

	private String saveFileName = "";
	private String saveFullFileName = "";
	private String savePath = "";

	private String encoding = "";
	private String defEncoding = System.getProperty("file.encoding");

	private MessageManager mm;

	private static ResourceBundle resources = ResourceBundle.getBundle("resources.jlighteditor");
	public static final String LINE_SEPARATOR = System.getProperty("line.separator");
	public static final String USER_HOME = System.getProperty("user.home");

	/**
	 * FileManager饹󥹥ȥ饯
	 * @param sh
	 */
	public FileManager(Shell sh) {
		shell = sh;
		encoding = defEncoding;
		mm = new MessageManager(sh);
	}

	/**
	 * եɽեString֤
	 * 󥻥ե뤬¸ߤʤnull
	 * @return String
	 */
	public String getOpenFileString() {
		final String textString;

		File file = getOpenFile();
		if (file == null || !file.isFile()) return null;

		FileInputStream fis = null;
		InputStreamReader isr = null;
		BufferedReader bfr = null;
		try {
			fis = new FileInputStream(file.getPath());
			isr = new InputStreamReader(fis, encoding);
			bfr = new BufferedReader(isr);

			StringBuffer buffer = new StringBuffer();
			String line = "";
//			System.out.println("encode = " + isr.getEncoding());

			while((line = bfr.readLine()) != null){
				if (!line.equals("")){
					//
					buffer.append(line);
				}
				buffer.append(LINE_SEPARATOR);
			}

			textString = buffer.toString();

			bfr.close();
			isr.close();

		} catch (FileNotFoundException e) {
			String message = MessageFormat.format(
				   resources.getString("Err_File_Not_Found"),
				   new String[] {file.getName()});
			mm.displayError(message);
			return null;

		} catch (IOException e) {
			String message = MessageFormat.format(
				   resources.getString("Err_File_IO"),
				   new String[] {file.getName()});
			mm.displayError(message);
			return null;

		} finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (IOException e) {
					String message = MessageFormat.format(
						   resources.getString("Err_File_IO"),
						   new String[] {file.getName()});
					mm.displayError(message);
					return null;
				}
			}
		}
		setFileStrings(file);

		return textString;
	}

	/**
	 * getOpenFileStringΥСɥ᥽å
	 * ե͡ǥեꤷStringǥեƤ֤
	 * ե뤬¸ߤʤnull顼ϡSystem.errؽ
	 * @return String
	 */
	public String getOpenFileString(String fullName) {

		final String textString;

		File file = new File(fullName);
		if (file == null || !file.isFile()) return null;

		FileInputStream fis = null;
		InputStreamReader isr = null;
		BufferedReader bfr = null;
		try {
			fis = new FileInputStream(file.getPath());
			isr = new InputStreamReader(fis, encoding);
			bfr = new BufferedReader(isr);

			StringBuffer buffer = new StringBuffer();
			String line = "";

			while((line = bfr.readLine()) != null){
				if (!line.equals("")){
					//
					buffer.append(line);
				}
				buffer.append(LINE_SEPARATOR);
			}

			textString = buffer.toString();

			bfr.close();
			isr.close();

		} catch (FileNotFoundException e) {
			String message = MessageFormat.format(
				   resources.getString("Err_File_Not_Found"),
				   new String[] {file.getName()});
			System.err.println(message);
			return null;

		} catch (IOException e) {
			String message = MessageFormat.format(
				   resources.getString("Err_File_IO"),
				   new String[] {file.getName()});
			System.err.println(message);
			return null;

		} finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (IOException e) {
					String message = MessageFormat.format(
						   resources.getString("Err_File_IO"),
						   new String[] {file.getName()});
					System.err.println(message);
					return null;
				}
			}
		}
		setFileStrings(file);

		return textString;
	}

	/**
	 * ϤFileե̾ե͡ࡢƥǥ쥯ȥ
	 * ѿ˥åȤ
	 * @param file
	 */
	private void setFileStrings(File file) {
		this.fileName = file.getName();
		this.fullFileName = file.getPath();
		this.path = file.getParent();
	}

	/**
	 * ե륪ץɽץ󤹤ե
	 * @return File
	 */
	private File getOpenFile() {

		FileDialog fileDialog = new FileDialog(shell, SWT.OPEN);
		fileDialog.setText(resources.getString("FileOpenDialogTitle"));

		File file = new File(path);
		if (file.exists()) {
			fileDialog.setFilterPath(path);
		} else {
			fileDialog.setFilterPath(USER_HOME);
		}
 		file = null;

		fileDialog.setFilterExtensions(new String[] {"*.java", "*.*"});
		fileDialog.open();

		String name = fileDialog.getFileName();

		// Canceled
		if ((name == null) || (name.length() == 0)) return null;

		file = new File(fileDialog.getFilterPath(), name);
		
		if (!file.exists()) {
			String message = MessageFormat.format(
				   resources.getString("Err_File_No_Exist"),
				   new String[] {file.getName()});
			mm.displayError(message);
			return null;
		}
		return file;
	}



	/**
	 * ƥȤ¸¸եե͡String֤
	 * 餫Υȥ֥¸ʤänull
	 * @return String
	 */
	public String saveText(String text) {
		String result = null;
		File file = new File(saveFullFileName);
		if (file.exists() && !file.canWrite()) {
			String message = MessageFormat.format(
					resources.getString("Err_File_Cannot_Write"),
					new String[] {file.getName()});
			mm.displayError(message);
			return null;
		}
		FileOutputStream fos = null;
		OutputStreamWriter osw = null;
		BufferedWriter bfw = null;
		try {
			fos = new FileOutputStream(file);
			osw = new OutputStreamWriter(fos, encoding);
			bfw = new BufferedWriter(osw);
			bfw.write(text);

			result = saveFullFileName;

			bfw.close();
			osw.close();

		} catch (FileNotFoundException e) {
			System.err.println("saveText FileNotFoundException !!!");
			String message = MessageFormat.format(
				   resources.getString("Err_File_Permission"),
				   new String[] {file.getName()});
			mm.displayError(message);
			return null;

		} catch (IOException e) {
			String message = MessageFormat.format(
				   resources.getString("Err_File_IO"),
				   new String[] {file.getName()});
			mm.displayError(message);
			return null;

		} finally {
			if (fos != null) {
				try {
					fos.close();
				} catch (IOException e) {
					String message = MessageFormat.format(
						   resources.getString("Err_File_IO"),
						   new String[] {file.getName()});
					mm.displayError(message);
					return null;
				}
			}
		}

		System.out.println(result);

		return result;
	}

	/**
	 * ̾դ¸ܥåɽƥȤ¸
	 * ¸ʤänull
	 * @param text
	 * @return String
	 */
	public String saveAsText(String text) {
		String result = null;

		FileDialog fileDialog = new FileDialog(shell, SWT.SAVE);
		fileDialog.setText(resources.getString("FileSaveDialogTitle"));

		File file = new File(savePath);
		if (file.exists()) {
			fileDialog.setFilterPath(savePath);
		} else {
			fileDialog.setFilterPath(USER_HOME);
		}
		file = null;

		fileDialog.setFileName(saveFileName);

		String name = fileDialog.open();

		// Canceled
		if ((name == null) || (name.length() == 0)) return null;


		file = new File(name);

		if (file.exists()) {
			String message = resources.getString("Qusetion.File.ReWrite");
			if (mm.displayYesNoQuestion(message) == SWT.NO) return null;
		} else {

			System.out.println("ե¸ߤʤ " + file.getPath());

		}

		// 򤵤줿˥ե̾Ϣ
		// ѿˤ줾ͤ
		saveFileName = fileDialog.getFileName();
		saveFullFileName = name;
		savePath = fileDialog.getFilterPath();

		result = saveText(text);


		return result;
	}


	/**
	 * ե̾
	 * @return String
	 */
	public String getFileName() {
		return fileName;
	}
	/**
	 * եΥե͡
	 * @return String
	 */
	public String getFullName() {
		return fullFileName;
	}
	/**
	 * եγǼƤǥ쥯ȥ
	 * @return String
	 */
	public String getPath() {
		return path;
	}


	/**
	 * ¸ե̾ꤹ
	 * @param saveFileName
	 */
	public void setSaveFileName(String saveFileName) {
		if (saveFileName != null) this.saveFileName = saveFileName;
	}
	/**
	 * ¸ե̾Υե͡ꤹ
	 * @param saveFullName
	 */
	public void setSaveFullName(String saveFullName) {
		if (saveFullName != null) this.saveFullFileName = saveFullName;
	}
	/**
	 * ¸ե̾γǼƤǥ쥯ȥ
	 * @param savePath
	 */
	public void setSavePath(String savePath) {
		if (savePath != null) this.savePath = savePath;
	}

	/**
	 * ¸ե̾
	 * @return String
	 */
	public String getSavedFileName() {
		return saveFileName;
	}
	/**
	 * ¸ե̾Υե͡
	 * @return String
	 */
	public String getSavedFullName() {
		return saveFullFileName;
	}
	/**
	 * ¸եγǼƤǥ쥯ȥ
	 * @return String
	 */
	public String getSavedPath() {
		return savePath;
	}

	/**
	 * 󥳡ǥ󥰤ꤹ
	 * @param encode
	 */
	public void setEncoding(String encode) {
		if (encode == null || encode == "") encode = defEncoding;
		encoding = encode;
	}

	/**
	 * Ϥե뤬¸ߤ뤫ɤĴ٤
	 * @param fullName
	 * @return boolean
	 */
	public boolean isFileExists(String fullName) {
		if (fullName == null || fullName == "") return false;
		File file = new File(fullName);
		if (file.exists()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Ϥե뤬եǤ뤫ɤĴ٤
	 * @param fullName
	 * @return boolean
	 */
	public boolean isFile(String fullName) {
		if (fullName == null || fullName == "") return false;
		File file = new File(fullName);
		if (file.isFile()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Ϥե뤬񤭹߲ǽɤĴ٤
	 * @param fullName
	 * @return boolean
	 */
	public boolean fileCanWrite(String fullName) {
		if (fullName == null || fullName == "") return false;
		File file = new File(fullName);
		if (file.canWrite()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * ե뤬ѹƤݤΥåɽ
	 * @param fileName
	 * @return int
	 */
	public int messageFileChanged(String fileName) {
		String message = MessageFormat.format(
			   resources.getString("Question.File.Changed"),
			   new String[] {fileName});
		return mm.displayYesNoCancel(message);
	}

}
